/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   copyright (C) 2004-2020                                               *
 *   Umbrello UML Modeller Authors <umbrello-devel@kde.org>                *
 ***************************************************************************/

#ifndef CODEACCESSORMETHODLIST_H
#define CODEACCESSORMETHODLIST_H

#include <QList>

// forward declarations
class CodeAccessorMethod;

typedef QList<CodeAccessorMethod*> CodeAccessorMethodList;
typedef QListIterator<CodeAccessorMethod*> CodeAccessorMethodListIt;

#endif
